#include <gtk/gtk.h>
#include "gigs_processor.h"
#include "gstreamer_player.h"

// Structure to hold GTK widgets for the form
typedef struct {
    GtkWidget *name_entry;
    GtkWidget *src_entry;
    GtkWidget *video_src_entry;
    GtkWidget *schedule_year_entry;
    GtkWidget *schedule_month_entry;
    GtkWidget *schedule_day_entry;
    GtkWidget *ical_src_entry;
    GtkWidget *location_src_entry;
    GtkWidget *location_text_entry;
    GigData *gig_data;  // Holds the parsed gig data
} GigForm;

// Function to save the data back to the .gig file
void save_gig_file(GtkWidget *widget, GigForm *form) {
    // Open a file chooser dialog
    GtkWidget *dialog = gtk_file_chooser_dialog_new("Save As...",
                                                    GTK_WINDOW(gtk_widget_get_toplevel(widget)),
                                                    GTK_FILE_CHOOSER_ACTION_SAVE,
                                                    "_Cancel", GTK_RESPONSE_CANCEL,
                                                    "_Save", GTK_RESPONSE_ACCEPT,
                                                    NULL);

    // Show the dialog and wait for user response
    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
        char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

        // Update the GigData struct with form data
        const char *name = gtk_entry_get_text(GTK_ENTRY(form->name_entry));
        const char *src = gtk_entry_get_text(GTK_ENTRY(form->src_entry));
        const char *video_src = gtk_entry_get_text(GTK_ENTRY(form->video_src_entry));
        const char *year = gtk_entry_get_text(GTK_ENTRY(form->schedule_year_entry));
        const char *month = gtk_entry_get_text(GTK_ENTRY(form->schedule_month_entry));
        const char *day = gtk_entry_get_text(GTK_ENTRY(form->schedule_day_entry));
        const char *ical_src = gtk_entry_get_text(GTK_ENTRY(form->ical_src_entry));
        const char *location_src = gtk_entry_get_text(GTK_ENTRY(form->location_src_entry));
        const char *location_text = gtk_entry_get_text(GTK_ENTRY(form->location_text_entry));

        // Update the GigData struct with form data
        form->gig_data->name = strdup(name);
        form->gig_data->src = strdup(src);
        form->gig_data->video_src = strdup(video_src);
        form->gig_data->schedule_url = strdup(ical_src);
        form->gig_data->location_url = strdup(location_src);
        form->gig_data->location_text = strdup(location_text);

        // Create a new XML document
        xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
        xmlNodePtr root_node = xmlNewNode(NULL, BAD_CAST "gig");
        xmlDocSetRootElement(doc, root_node);

        // <musician>
        xmlNodePtr musician_node = xmlNewChild(root_node, NULL, BAD_CAST "musician", NULL);
        xmlNewProp(musician_node, BAD_CAST "name", BAD_CAST form->gig_data->name);
        xmlNewProp(musician_node, BAD_CAST "src", BAD_CAST form->gig_data->src);

        // <video>
        xmlNodePtr video_node = xmlNewChild(musician_node, NULL, BAD_CAST "video", NULL);
        xmlNewProp(video_node, BAD_CAST "src", BAD_CAST form->gig_data->video_src);

        // <schedule>
        xmlNodePtr schedule_node = xmlNewChild(root_node, NULL, BAD_CAST "schedule", NULL);
        xmlNewProp(schedule_node, BAD_CAST "year", BAD_CAST year);
        xmlNewProp(schedule_node, BAD_CAST "month", BAD_CAST month);
        xmlNewProp(schedule_node, BAD_CAST "day", BAD_CAST day);

        // <ical>
        xmlNodePtr ical_node = xmlNewChild(schedule_node, NULL, BAD_CAST "ical", NULL);
        xmlNewProp(ical_node, BAD_CAST "src", BAD_CAST form->gig_data->schedule_url);

        // <location>
        xmlNodePtr location_node = xmlNewChild(schedule_node, NULL, BAD_CAST "location", BAD_CAST form->gig_data->location_text);
        xmlNewProp(location_node, BAD_CAST "src", BAD_CAST form->gig_data->location_url);

        // Write to a temporary file
        char *temp_file = "example.tmp.gig";
        if (xmlSaveFormatFileEnc(temp_file, doc, "UTF-8", 1) == -1) {
            fprintf(stderr, "Error saving temporary file\n");
            xmlFreeDoc(doc);
            g_free(filename);
            return;
        }
        xmlFreeDoc(doc);

        // Rename the temporary file to the user-specified filename
        if (rename(temp_file, filename) != 0) {
            fprintf(stderr, "Error renaming temporary file to %s\n", filename);
            remove(temp_file);
            g_free(filename);
            return;
        }

        // Inform the user
        GtkWidget *message_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "Gig file saved successfully!");
        gtk_dialog_run(GTK_DIALOG(message_dialog));
        gtk_widget_destroy(message_dialog);
        
        // Free the allocated filename
        g_free(filename);
    }

    // Destroy the file chooser dialog
    gtk_widget_destroy(dialog);
}

// Function to populate the form with data from GigData
void populate_form(GigForm *form) {
    gtk_entry_set_text(GTK_ENTRY(form->name_entry), form->gig_data->name);
    gtk_entry_set_text(GTK_ENTRY(form->src_entry), form->gig_data->src);
    gtk_entry_set_text(GTK_ENTRY(form->video_src_entry), form->gig_data->video_src);

    // For simplicity, assuming schedule as strings (year, month, day)
    gtk_entry_set_text(GTK_ENTRY(form->schedule_year_entry), "2025");
    gtk_entry_set_text(GTK_ENTRY(form->schedule_month_entry), "02");
    gtk_entry_set_text(GTK_ENTRY(form->schedule_day_entry), "20");

    gtk_entry_set_text(GTK_ENTRY(form->ical_src_entry), form->gig_data->schedule_url);
    gtk_entry_set_text(GTK_ENTRY(form->location_src_entry), form->gig_data->location_url);
    gtk_entry_set_text(GTK_ENTRY(form->location_text_entry), form->gig_data->location_text);
}

// Main function to create the GTK form interface
int main(int argc, char *argv[]) {
    gtk_init(&argc, &argv);

    // Load the gig file and parse it
    GigForm form;
    form.gig_data = parse_gig_file("example.gigs");

    // Create the window
    GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "Edit Gig File");
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
    gtk_window_set_default_size(GTK_WINDOW(window), 400, 300);
    g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

    // Create a grid to arrange form fields
    GtkWidget *grid = gtk_grid_new();
    gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
    gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
    gtk_container_add(GTK_CONTAINER(window), grid);

    // Add form labels and entries
    GtkWidget *name_label = gtk_label_new("Name:");
    form.name_entry = gtk_entry_new();
    GtkWidget *src_label = gtk_label_new("Source URL:");
    form.src_entry = gtk_entry_new();
    GtkWidget *video_src_label = gtk_label_new("Video URL:");
    form.video_src_entry = gtk_entry_new();
    GtkWidget *schedule_year_label = gtk_label_new("Schedule Year:");
    form.schedule_year_entry = gtk_entry_new();
    GtkWidget *schedule_month_label = gtk_label_new("Schedule Month:");
    form.schedule_month_entry = gtk_entry_new();
    GtkWidget *schedule_day_label = gtk_label_new("Schedule Day:");
    form.schedule_day_entry = gtk_entry_new();
    GtkWidget *ical_src_label = gtk_label_new("iCal URL:");
    form.ical_src_entry = gtk_entry_new();
    GtkWidget *location_src_label = gtk_label_new("Location URL:");
    form.location_src_entry = gtk_entry_new();
    GtkWidget *location_text_label = gtk_label_new("Location Text:");
    form.location_text_entry = gtk_entry_new();

    // Attach widgets to the grid
    gtk_grid_attach(GTK_GRID(grid), name_label, 0, 0, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.name_entry, 1, 0, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), src_label, 0, 1, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.src_entry, 1, 1, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), video_src_label, 0, 2, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.video_src_entry, 1, 2, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), schedule_year_label, 0, 3, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.schedule_year_entry, 1, 3, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), schedule_month_label, 0, 4, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.schedule_month_entry, 1, 4, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), schedule_day_label, 0, 5, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.schedule_day_entry, 1, 5, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), ical_src_label, 0, 6, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.ical_src_entry, 1, 6, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), location_src_label, 0, 7, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.location_src_entry, 1, 7, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), location_text_label, 0, 8, 1, 1);
    gtk_grid_attach(GTK_GRID(grid), form.location_text_entry, 1, 8, 1, 1);

    // Add a save button
    GtkWidget *save_button = gtk_button_new_with_label("Save");
    g_signal_connect(save_button, "clicked", G_CALLBACK(save_gig_file), &form);
    gtk_grid_attach(GTK_GRID(grid), save_button, 1, 9, 1, 1);

    // Populate form with existing data
    populate_form(&form);

    // Show everything
    gtk_widget_show_all(window);

    // Run the GTK main loop
    gtk_main();

    // Free parsed data
    free_gig_data(form.gig_data);

    return (0);
}
